luanet.load_assembly("System");

local types = {};
types["Regex"] = luanet.import_type("System.Text.RegularExpressions.Regex");

function NavigateToReceiptPage()
	local browser = GetBrowser();

	local aCollection = browser.Document:GetElementsByTagName("a");
	LogDebug("Count of Anchor tags is: " .. aCollection.Count);
	local receiptButton = nil;

	local aCollectionEnumerator = aCollection:GetEnumerator();
	while aCollectionEnumerator:MoveNext() do
		local element = aCollectionEnumerator.Current;
		if element:GetAttribute("className") ~= "" then
			LogDebug("Found anchor class: " ..element:GetAttribute("className"));
			if string.match(element:GetAttribute("className"), "btn%-primary") then
				receiptButton = element;
				break;
			end
		end
	end

	local link = receiptButton:GetAttribute("href");
	Navigate(link);
end

function OnPageByTitle(titleString)
	LogDebug("Checking if on page by title: "..titleString);

	local onPage = false;

	local browser = GetBrowser();
	local titleCollection = browser.Document:GetElementsByTagName("Title");
	LogDebug("Count of Title tags is: " .. titleCollection.Count);

	local titleCollectionEnumerator = titleCollection:GetEnumerator();
	while titleCollectionEnumerator:MoveNext() do
		local element = titleCollectionEnumerator.Current;
		LogDebug("Found Title: " .. element.InnerText);
		if string.match(element.InnerText,titleString) then
			LogDebug("Title matches");
			onPage = true;
			break;
		end
	end

	if onPage then
		LogDebug("On ("..titleString..") page: true");
	else
		LogDebug("On ("..titleString..") page: false");
	end

	return onPage;
end

function OnDoclineHomePage()
	return OnPageByTitle("DOCLINE.+Home");
end

function OnPMIDArticleSelectionPage()
	return OnPageByTitle("PMID Request.+Select Articles");
end

function OnPMIDSearchPage()
	return OnPageByTitle("Docline PMID Order.+Find a PMID") or OnPageByTitle("PMID Request.+Find Citations");
end

function OnPMIDOrderPage()
	return OnPageByTitle("Docline PMID Order.+Place Order") or OnPageByTitle("[.*%s]?PMID Request.+Place Order");
end

function OnManualOrderPage()
	return OnPageByTitle("Docline Manual Order.+Place Order") or OnPageByTitle("Manual Request.+Place Order");
end

function OnSuccessPage()
	return OnPageByTitle("Docline PMID Order.+Success") or OnPageByTitle("PMID Request.+Results") or OnPageByTitle("Manual.+Success");
end

function OnDoclineRequestPage()
	return OnPageByTitle("Docline Request");
end

function NavigateToPlaceRequest()
	LogDebug("Navigating to Find a PMID form.");
	local browser = GetBrowser();
	if (browser.Document:GetElementById("nav-item-borrow") ~= nil) then
		NavigateById("nav-borrow-request-pmids");
	end
end

function SearchPMID()
	local browser = GetBrowser();

	local pubmedIdElement = browser.Document:GetElementById("id_pmids");

	local pmid = FieldTranslator.GetTranslationResults(FieldTranslator, "PubMedSearchID").Value;

	pubmedIdElement.SetAttribute(pubmedIdElement, "value", pmid);
	if pmid ~= "" then
		browser.Document:GetElementById("submit-id-submit"):InvokeMember("click");
	end
end

function SelectArticleIfOnlyOne()
	--[[
		When on the "PMID Request - Select Articles" page, selects the
		"Select all" button and then the "Next" button if there is only one
		record in the "Citation Information" list.
	]]

	LogDebug("Checking to see if the 'PMID Request.+Select Articles' only has 1 result");

	local browser = GetBrowser();

	-- Builds an array containing all of the elements on the page that have a class
	-- name of pmid-order-journal-article-info, which should be equivalent to the
	-- number of results on the page.

	local pmidOrderJournalArticleInfoDivs = {};

	local selectArticlesForm = browser.Document:GetElementById("select-articles");
	local selectArticlesFormDivs = selectArticlesForm:GetElementsByTagName("div");
	local selectArticlesFormDivsEnumerator = selectArticlesFormDivs:GetEnumerator();
	while selectArticlesFormDivsEnumerator:MoveNext() do
		local element = selectArticlesFormDivsEnumerator.Current;
		if element:GetAttribute("className") ~= "" then
			LogDebug("Found class: " .. element:GetAttribute("className"));
			if string.match(element:GetAttribute("className"), "pmid%-order%-journal%-article%-info") then
				table.insert(pmidOrderJournalArticleInfoDivs, element);
			end
		end
	end

	LogDebug("Number of results: " .. #pmidOrderJournalArticleInfoDivs);

	if #pmidOrderJournalArticleInfoDivs == 1 then
		-- Look for duplicate warning -- "<h4 class="alert-heading">Previously Ordered PMID</h4>"
		-- Look for Free full-text may be available (e.g. PMID 33810229) -- "<h4 class="alert-heading">Free full-text may be available</h4>"
		LogDebug("Looking for a duplicate submission or if available via open access");
		if (string.match(selectArticlesForm.InnerText, "Previously Ordered PMID") or
			string.match(selectArticlesForm.InnerText, "Free full%-text may be available")) then
			LogDebug("Found a previously ordered request or may be available as open access. Cannot auto-advance the page.");
		else
			LogDebug("Duplicate request not found");

			LogDebug("Clicking \"Select All\"...");
			browser.Document:GetElementById("select-all-checkboxes"):InvokeMember("click");
			LogDebug("Clicking \"Next\"...");
			browser.Document:GetElementById("submit-id-submit"):InvokeMember("click");
		end
	else
		LogDebug("More than one result detected, cannot auto-advance the page.");
	end
end

function PopulateSearchForm()
	LogDebug("Populating Search Form");
	local browser = GetBrowser();
	local currentValue = nil;

	--Authorized By
	-- This is autofilled in by the Docline login information. We are overriding it.
	browser.Document:GetElementById("id_authorized_by").SetAttribute(browser.Document:GetElementById("id_authorized_by"), "value", FieldTranslator.GetTranslationResults(FieldTranslator, "AuthorizedBy").Value);

	--Max Cost
	-- This is autofilled in by the Docline login information. Only override if one is present
	local maxCost = FieldTranslator.GetTranslationResults(FieldTranslator, "MaxCost").Value;
	if maxCost ~= nil then
		browser.Document:GetElementById("id_max_cost").SetAttribute(browser.Document:GetElementById("id_max_cost"), "value", FieldTranslator.GetTranslationResults(FieldTranslator, "MaxCost").Value);
	end

	--Patron Name
	browser.Document:GetElementById("id_patron_name").SetAttribute(browser.Document:GetElementById("id_patron_name"), "value", FieldTranslator.GetTranslationResults(FieldTranslator, "PatronName").Value);

	--Need By Date
	local needBy = FieldTranslator.GetTranslationResults(FieldTranslator, "NeedBy").Value;
	if needBy ~= nil and needBy ~= "" then
		browser.Document:GetElementById("id_need_by_date").SetAttribute(browser.Document:GetElementById("id_need_by_date"), "value", needBy);
	end

	--Copyright Compliance
	local copyrightComp = FieldTranslator.GetTranslationResults(FieldTranslator, "CopyrightCompliance").Value;
	if copyrightComp ~= nil and copyrightComp ~= "" then
		browser.Document:GetElementById("id_copyright_compliance").SetAttribute(browser.Document:GetElementById("id_copyright_compliance"), "value", copyrightComp);
	end

	--Service Level
	local serviceLevel = FieldTranslator.GetTranslationResults(FieldTranslator, "ServiceLevel").Value;
	if serviceLevel ~= nil and serviceLevel ~= "" then
		browser.Document:GetElementById("id_service_level").SetAttribute(browser.Document:GetElementById("id_service_level"), "value", serviceLevel);
	end

	--Service Type
	local serviceType = FieldTranslator.GetTranslationResults(FieldTranslator, "ServiceType").Value;
	if serviceType ~= nil and serviceType ~= "" then
		browser.Document:GetElementById("id_request_type_requested").SetAttribute(browser.Document:GetElementById("id_request_type_requested"), "value", serviceType);
	end

	--Delivery Method
	local deliveryMethod = FieldTranslator.GetTranslationResults(FieldTranslator, "DeliveryMethod").Value;
	if deliveryMethod ~= nil and deliveryMethod ~= "" then
		browser.Document:GetElementById("id_delivery_method_requested").SetAttribute(browser.Document:GetElementById("id_delivery_method_requested"), "value", deliveryMethod);
	end

	--Odyssey
	browser.Document:GetElementById("id_delivery_odyssey").SetAttribute(browser.Document:GetElementById("id_delivery_odyssey"), "value", FieldTranslator.GetTranslationResults(FieldTranslator, "OdysseySystemID").Value);
end

function PopulateManualForm()
	PopulateSearchForm();

	LogDebug("Populating Manual Form");
	local browser = GetBrowser();

	--Journal/Book Title (required)
	browser.Document:GetElementById("id_journal_title").SetAttribute(browser.Document:GetElementById("id_journal_title"), "value", FieldTranslator.GetTranslationResults(FieldTranslator, "Title").Value);

	--Book Author/Editor
	browser.Document:GetElementById("id_book_author").SetAttribute(browser.Document:GetElementById("id_book_author"), "value", FieldTranslator.GetTranslationResults(FieldTranslator, "JournalAuthor").Value);

	--Article or Chapter Title
	browser.Document:GetElementById("id_article_title").SetAttribute(browser.Document:GetElementById("id_article_title"), "value", FieldTranslator.GetTranslationResults(FieldTranslator, "Article").Value);

	--Article or Chapter Author
	browser.Document:GetElementById("id_article_authors").SetAttribute(browser.Document:GetElementById("id_article_authors"), "value", FieldTranslator.GetTranslationResults(FieldTranslator, "Author").Value);

	--Publisher
	browser.Document:GetElementById("id_journal_publisher").SetAttribute(browser.Document:GetElementById("id_journal_publisher"), "value", FieldTranslator.GetTranslationResults(FieldTranslator, "Publisher").Value);

	--Place of Publication
	browser.Document:GetElementById("id_journal_place_of_publication").SetAttribute(browser.Document:GetElementById("id_journal_place_of_publication"), "value", FieldTranslator.GetTranslationResults(FieldTranslator, "PubPlace").Value);

	--ISSN/ISBN
	browser.Document:GetElementById("id_journal_issn").SetAttribute(browser.Document:GetElementById("id_journal_issn"), "value", FieldTranslator.GetTranslationResults(FieldTranslator, "Issn").Value);

	--Year (required)
	browser.Document:GetElementById("id_article_year_string").SetAttribute(browser.Document:GetElementById("id_article_year_string"), "value", FieldTranslator.GetTranslationResults(FieldTranslator, "PubYear").Value);

	--Volume
	browser.Document:GetElementById("id_article_volume_string").SetAttribute(browser.Document:GetElementById("id_article_volume_string"), "value", FieldTranslator.GetTranslationResults(FieldTranslator, "Volume").Value);

	--Issue
	browser.Document:GetElementById("id_article_issue").SetAttribute(browser.Document:GetElementById("id_article_issue"), "value", FieldTranslator.GetTranslationResults(FieldTranslator, "Issue").Value);

	--Pages
	browser.Document:GetElementById("id_article_pages").SetAttribute(browser.Document:GetElementById("id_article_pages"), "value", FieldTranslator.GetTranslationResults(FieldTranslator, "Pages").Value);

	--Patron Name
	browser.Document:GetElementById("id_patron_name").SetAttribute(browser.Document:GetElementById("id_patron_name"), "value", FieldTranslator.GetTranslationResults(FieldTranslator, "PatronName").Value);
end

function SubmitRequest()
	local browser = GetBrowser();
	local submitButton = browser.Document:GetElementById("submit-id-submit");
	if (submitButton ~= nil) then
		submitButton:InvokeMember("click");
	else
		LogDebug("Submit Button not found on page");
	end
end

function Finished()

	_receiptTable["BorrowerTN"] = FieldTranslator.GetTranslationResults(FieldTranslator, "BorrowerTN").Value;

	NavigateToReceiptPage();
end

FieldTranslator = GetFieldTranslator()

RegisterPageHandler("custom", "OnSuccessPage", "Finished", true)
RegisterPageHandler("custom", "OnDoclineRequestPage", "ImportReceiptPage", true)
