require "Docline\\MainDocline";

local _importingActive = false;
local _markReceipted = false;
local _receiptLinksToProcess = {};

local _importAllButton;
local _importCurrentButton;

local _types = {};
_types["System.Object"] = luanet.import_type("System.Object");

Navigate(DoclineUrl);

function InjectScript(destination, script)
	local objArray = _types["System.Object"][2];
	objArray[0] = script;
	objArray[1] = "JavaScript";
	destination:InvokeScript("execScript", objArray);
end

function Initialize(importAllButton, importCurrentButton)
	_importAllButton = importAllButton;
	_importCurrentButton = importCurrentButton;

	RegisterPageHandler("custom", "IsNavMenuLoaded", "NavigateToLendingReceipts", false);

	--Critical so that it happens on every navigation
	RegisterPageHandler("custom", "IsReceiptForm", "ReceiptFormLoaded", true);
	RegisterPageHandler("custom", "HasLinkToRequest", "RemoveNewWindowLinkBehavior", true);
	RegisterPageHandler("custom", "HasPrintButton", "RemoveNewWindowPrintBehavior", true);
end

function NavigateHome()
	Navigate(DoclineUrl);
end

function IsNavMenuLoaded()
	LogDebug("IsNavMenuLoaded");
	local browser = GetBrowser();
	return (browser.Document:GetElementById("nav-item-lend") ~= nil)
end

function NavigateToLendingReceipts()
	LogDebug("Form has nav menu. Navigating to Lending Receipt requests form.");
	NavigateById("nav-lend-receipt")
end

function IsReceiptForm()
	LogDebug("Checking if on receipt form");

	local onReceiptForm = false;

	local browser = GetBrowser();
	local tableCollection = browser.Document:GetElementsByTagName("TABLE");
	LogDebug("Count of Table tags is: " .. tableCollection.Count);

	local tableCollectionEnumerator = tableCollection:GetEnumerator();
	while tableCollectionEnumerator:MoveNext() do
		local element = tableCollectionEnumerator.Current;
		LogDebug("Found TABLE class: " .. element:GetAttribute("className"));
		if element:GetAttribute("className") == "request-table" then
			onReceiptForm = true;
			break;
		end
	end

	LogDebug("On Receipt form: " .. tostring(onReceiptForm));

	--Update the button states in the check handler so they will be updated with other navigation
	if (_importCurrentButton) then
		_importCurrentButton.Enabled = onReceiptForm;
	else
		LogInfo("Import Current button is not available");
	end

	return onReceiptForm;
end

function ReceiptFormLoaded()
	--Critical so that it happens on every navigation
	RegisterPageHandler("custom", "IsReceiptForm", "ReceiptFormLoaded", true);

	if (_importingActive == true) then
		LogInfo("On Receipt Form and Importing is Active. Attempting to start import.");

		local requestsToImport = GetRequestsToImport();
		ImportRequestsFromCurrentPage(requestsToImport);
	else
		LogDebug("Importing is not active");
	end
end


function HasLinkToRequest()
	LogDebug("Checking for request links that would be opened in a new window");

	local hasLinkToRequest = false;

	local browser = GetBrowser();
	local linkCollection = browser.Document:GetElementsByTagName("div");

	--Find any divs of class "result"
	local linkCollectionEnumerator = linkCollection:GetEnumerator();
	while linkCollectionEnumerator:MoveNext() do
		local element = linkCollectionEnumerator.Current;
		if (element:GetAttribute("className") == "results") then
			hasLinkToRequest = true;
			break;
		end
	end

	LogDebug("Has link to request: " .. tostring(hasLinkToRequest));

	return hasLinkToRequest;
end

function HasPrintButton()
	LogDebug("Checking for print buttons that would open in a new window");

	local printButton = GetBrowser().Document:GetElementById("print-selected");

	local hasPrintButton = (printButton ~= nil);

	LogDebug("Has print button: " .. tostring(hasPrintButton));

	return hasPrintButton;
end

function RemoveNewWindowLinkBehavior()
	--Critical so that it happens on every navigation
	RegisterPageHandler("custom", "HasLinkToRequest", "RemoveNewWindowLinkBehavior", true);

	local browser = GetBrowser();
	local linkCollection = browser.Document:GetElementsByTagName("div");

	--Find any divs of class "result"
	local linkCollectionEnumerator = linkCollection:GetEnumerator();
	while linkCollectionEnumerator:MoveNext() do
		local element = linkCollectionEnumerator.Current;
		if (element:GetAttribute("className") == "results") then
			RetargetNewWindowLinksInDiv(element);
		end
	end
end

function RemoveNewWindowPrintBehavior()
	RegisterPageHandler("custom", "HasPrintButton", "RemoveNewWindowPrintBehavior", true);

	local script = [[(function() {
            function get_all_selected_ids_as_list(){
                var id_list = [];
                $('input.request-action-checkbox:checked').each(function () {
                    var checkbox_id = $(this).data('request-pk');
                    id_list.push(checkbox_id);
                });
                return id_list;
            }
            function create_id_list_variable(id_list){
                return id_list.join(',');
            }

			$('button#print-selected').unbind("click").click(function (e) {
                var url = $(this).data('print-url');
                var id_list = get_all_selected_ids_as_list();
                var id_variable_string = create_id_list_variable(id_list);
                var full_url = url + id_variable_string;
                var win = window.open(full_url, '_self');
				e.stopImmediatePropagation();
            })})();
		]];

	InjectScript(GetBrowser().Document, script);
end

function RetargetNewWindowLinksInDiv(resultsDiv)
	local anchors = resultsDiv:GetElementsByTagName("a");

	--Find any links that would open in a new window and remove their target attribute
	local anchorsEnumerator = anchors:GetEnumerator();
	while anchorsEnumerator:MoveNext() do
		local anchor = anchorsEnumerator.Current;
		if (anchor:GetAttribute("target") == "_blank") then
			LogDebug("Replacing a.target attribute with '_self'");
			anchor:SetAttribute("target", "_self");		--Change the link to open in the current window
		end
	end
end

function IsGeneralLendingReceiptRequestsPage()
	local browser = GetBrowser();
	LogDebug("IsGeneralLendingReceiptRequestsPage: "..browser.DocumentTitle);
	return (string.match(browser.DocumentTitle, "New Lend Requests") ~= nil);
end

function ImportAllRequestsHandler()
	--Re-enable the handler
	RegisterPageHandler("custom", "IsGeneralLendingReceiptRequestsPage", "ImportAllRequestsHandler", true);

	if (_importingActive == true) then
		ImportAll();
	end


end

function ImportAll()
	-- Enable the check to see if we are on the main receipts page so the importing
	-- process can kick off automatically after navigation is complete
	-- This is to start importing when we redirect or if we are marking items receipted
	RegisterPageHandler("custom", "IsGeneralLendingReceiptRequestsPage", "ImportAllRequestsHandler", true);

	local browser = GetBrowser();

	LogDebug("Attempting to import all requests");
	--Initialize importing to automate page clicks
	_importingActive = true;
	_markReceipted = true;

	--If not on the general receipts page, navigate to it
	if (not string.match(browser.DocumentTitle, "New Lend Requests")) then
		LogDebug("Current page is not the general lend requests page. Navigating to lending receipts.");

		if (IsNavMenuLoaded) then
			--Nav menu is already loaded so click to lending receipts
			NavigateToLendingReceipts();
		else
			--If the nav menu isn't found, redirect to the main docline page and re-enable the initial handler
			RegisterPageHandler("custom", "IsNavMenuLoaded", "NavigateToLendingReceipts", false);
			Navigate(DoclineUrl);
		end
	else
		LogDebug("On General Receipts Page");
		if (#_receiptLinksToProcess == 0) then
			_receiptLinksToProcess = GetReceiptLinks(browser);

			LogDebug("Found " .. #_receiptLinksToProcess .. " receipt links.");

			local totalRequestsToImport = 0;
			for i = 1, #_receiptLinksToProcess, 1 do
				LogDebug(_receiptLinksToProcess[i]["Link"]);
				totalRequestsToImport = totalRequestsToImport + tonumber(_receiptLinksToProcess[i]["Count"]);
			end

			InitializeImport(totalRequestsToImport);
		end

		--If we are marking receipts, we set up a handler to re-navigate to lending receipts
		if (_markReceipted) then
			LogDebug("Registering page handlers to redirect to receipts page.");
			--If the nav menu isn't found, redirect to the main docline page and re-enable the initial handler
			RegisterPageHandler("custom", "IsNavMenuLoaded", "NavigateToLendingReceipts", false);
		else
			LogDebug("Skipping page handler registration.");
		end

		ProcessReceiptLinks();
	end
end

function GetReceiptLinks(browser)
	local receiptLinks = {};

	--Receipt links are found in the main section tag
	local sectionCollection = browser.Document:GetElementsByTagName("section");
	LogDebug("Count of Section tags is: " .. sectionCollection.Count);

	local requestCounts = {};

	--Look at all links in the section (there should only be the receipt links)
	local sectionCollectionEnumerator = sectionCollection:GetEnumerator();
	while sectionCollectionEnumerator:MoveNext() do
		local sectionElement = sectionCollectionEnumerator.Current;

		local divCollection = sectionElement:GetElementsByTagName("div");
		LogDebug("Count of div tags is: " .. divCollection.Count);
		local divCollectionEnumerator = divCollection:GetEnumerator();
		while divCollectionEnumerator:MoveNext() do
			local divElement = divCollectionEnumerator.Current;
			if (string.match(divElement:GetAttribute("className"), "form%-row")) then
				local count = 0;
				local type = nil;
				local h2Collection = divElement:GetElementsByTagName("h2");
				if (h2Collection.Count > 0) then
					local h2CollectionEnumerator = h2Collection:GetEnumerator();
					h2CollectionEnumerator:MoveNext();
					local firstH2 = h2CollectionEnumerator.Current;

					LogDebug("H2: " .. firstH2.InnerText);
					count, type = string.match(firstH2.InnerText, "(%d+) (.+) Request");
				end

				if (type ~= nil and tonumber(count) > 0) then
					LogDebug("Type: " .. type);
					LogDebug("Count: " .. count);

					--Initialize the value if not already found
					if (requestCounts[type] == nil) then
						LogDebug("Initializing "..type)
						requestCounts[type] = 0;
					end
					LogDebug("Incrementing"..type)
					requestCounts[type] = requestCounts[type] + count;

					LogDebug("Looking for links")
					local linkCollection = divElement:GetElementsByTagName("a");
					local linkCollectionEnumerator = linkCollection:GetEnumerator();
					while linkCollectionEnumerator:MoveNext() do
						local linkElement = linkCollectionEnumerator.Current;
						--look for a match of Receipt Request (singular) which should handle 1 or more requests
						if (string.match(linkElement.InnerText, "Receipt Request")) then
							table.insert(receiptLinks, { Type = type, Count = count, Link = linkElement:GetAttribute("href") });
						end
					end
				end
			end

		end
	end

	return receiptLinks;
end

function ProcessReceiptLinks()
	--If there are receipt links to handle, process the first link. Additional links will be handled
	-- in the page handlers when completed.
	if (#_receiptLinksToProcess > 0) then
		--We can do 1 navigate at a time so we are navigating to the first link to perform importing on
		local receiptLink = _receiptLinksToProcess[1];
		LogDebug("Navigating to Receipt Link for " .. receiptLink["Type"]);
		table.remove(_receiptLinksToProcess, 1);
		LogDebug(receiptLink["Link"]);
		Navigate(receiptLink["Link"]);
		LogDebug("Clicked " .. receiptLink["Type"]);
	else
		LogDebug("Completed handling all receipt links");

		_importingActive = false;
		_markReceipted = false;

		CompleteImport();
	end
end

function ImportCurrent()
	LogDebug("Attempting to import current requests");
	_importingActive = true;
	_markReceipted = false;

	local requestsToImport = GetRequestsToImport();
	InitializeImport(#requestsToImport);

	ImportRequestsFromCurrentPage(requestsToImport);
	_importingActive = false;
	_markReceipted = false;

	CompleteImport();
end

function ImportReceiptCurrent()
	LogDebug("Attempting to import and receipt current requests");
	_importingActive = true;
	_markReceipted = true;

	local requestsToImport = GetRequestsToImport();
	InitializeImport(#requestsToImport);
	ImportRequestsFromCurrentPage(requestsToImport);

	_importingActive = false;
	_markReceipted = false;

	CompleteImport();
end

function GetRequestsToImport()
	local browser = GetBrowser();
	local tableCollection = browser.Document:GetElementsByTagName("TABLE");
	LogDebug("Count of Table tags is: " .. tableCollection.Count);

	local requestElementsToProcess = {};

	local tableCollectionEnumerator = tableCollection:GetEnumerator();
	while tableCollectionEnumerator:MoveNext() do
		local element = tableCollectionEnumerator.Current;
		LogDebug("Found TABLE class: " .. element:GetAttribute("className"));

		if string.match(element:GetAttribute("className"), "request%-table") then
			--Process table
			table.insert(requestElementsToProcess, element);
		end
	end

	return requestElementsToProcess;
end

function ImportRequestsFromCurrentPage(requestElementsToProcess)
	local browser = GetBrowser();

	LogInfo("Importing requests from page: " .. browser.Url:ToString());

	for requestIndex, requestElement in pairs(requestElementsToProcess) do

		LogInfo("Importing Request " .. requestIndex);

		ParseReceiptPage(requestElement);
		DoTransactionImport(_receiptTable, requestElement.InnerText);
		_receiptTable = {};
	end

	if (_markReceipted == true) then
		LogInfo("Attempting to mark requests as receipted");
		local buttonElements = browser.Document:GetElementsByTagName("BUTTON");
		local buttonElementsEnumerator = buttonElements:GetEnumerator();
		while buttonElementsEnumerator:MoveNext() do
			local element = buttonElementsEnumerator.Current;
			LogDebug("Found BUTTON: " .. element.InnerText);
			if string.match(element.InnerText, "Mark All As Receipted and Return To DOCLINE") then
				LogInfo("Marking items receipted");
				element:InvokeMember("click");

				--Found the right button so break the loop & return
				return true;
			end
		end
	end
end
