require "Docline\\MainDocline";

local _buttonCancel  = nil;
local _buttonFill = nil;
local _statusItem = nil;

Navigate(DoclineUrl);

function NavigateHome()
	Navigate(DoclineUrl);
end

function NavigateToUpdateRequests(buttonCancel, buttonFill, statusItem)
	_buttonCancel = buttonCancel;
	_buttonFill = buttonFill;
	_statusItem = statusItem;

	RegisterPageHandler("custom", "IsReadyForBatchUpdate", "NavigateToBatchUpdate", false);
	RegisterPageHandler("custom", "IsBatchUpdateForm", "BatchUpdateFormLoaded", false);
end

function IsReadyForBatchUpdate()
	local browser = GetBrowser();
	return browser.Document:GetElementById("nav-item-lend") ~= nil;
end

function NavigateToBatchUpdate()
	LogDebug("Form has nav menu. Navigating to Batch update form.");
	local browser = GetBrowser();
	if (browser.Document:GetElementById("nav-lend-selected-update") ~= nil) then
        NavigateById ("nav-lend-selected-update");
    else
        NavigateById("nav-lend-update");
    end
end

function IsBatchUpdateForm()
	LogDebug("Checking if On then batch update Form");

	local browser = GetBrowser();
	local onUpdateForm = browser.Document:GetElementById("request-batch-update") ~= nil;

	if onUpdateForm then
		LogDebug("On Update form: true");
	else
		LogDebug("On Update form: false");
	end

	--Update the button states in the check handler so they will be updated with other navigation
	if (_buttonCancel) then
		_buttonCancel.Enabled = onUpdateForm;
	else
		LogInfo("Cancel button is not available");
	end

	if (_buttonFill) then
		_buttonFill.Enabled = onUpdateForm;
	else
		LogInfo("Fill button is not available");
	end;

	return onUpdateForm;
end

function BatchUpdateFormLoaded()
	--Re-Register the handler so buttons can be disabled if we move off the batch update form
	RegisterPageHandler("custom", "IsBatchUpdateForm", "BatchUpdateFormLoaded", false);
end

function UpdateRequestNumbersInput(requests)
	local browser = GetBrowser();
	local requestNumbersTextArea = browser.Document:GetElementById("id_request_id_list");
	if (requestNumbersTextArea ~= nil) then
		PasteMemoText(requestNumbersTextArea, requests);
	else
		LogInfo("Unable to find request numbers input area");
	end
end

function UpdateResultSelection(resultValue)
	local browser = GetBrowser();
	local filledOrRejectedDiv = browser.Document:GetElementById("div_id_filled_or_rejected");
	SelectInputOption(filledOrRejectedDiv, "input", "checked", resultValue)
end

function UpdateRejectionReason(resultValue)
	local browser = GetBrowser();
	local filledOrRejectedDiv = browser.Document:GetElementById("id_rejected_because_reason");
	SelectInputOption(filledOrRejectedDiv, "option", "selected", resultValue)
end

function UpdateCancelledRequests(reason, cancelledRequests)
	LogDebug("Cancelling requests");

	local success, result = pcall(function ()
		UpdateResultSelection("rejected");
		UpdateRejectionReason(GetRejectionReason(reason));
		UpdateRequestNumbersInput(cancelledRequests);
		return true;
  	end);

	if success then
		LogInfo("Successfully updated Docline batch update form with Cancelled requests.");
		if (_statusItem ~= nil) then
			_statusItem.Caption = "No Requests (" .. reason .. ") pasted into Docline window. Submit requests and proceed.";
		end
	else
		LogError("Error updating cancelled requests: " .. result);
		if (_statusItem ~= nil) then
			_statusItem.Caption = "An error occurred pasting data into docline window.";
		end
	end
end

function UpdateFilledRequests(filledRequests)
	LogDebug("Pasting yes requests");

	local success, result = pcall(function ()
		UpdateResultSelection("filled_as_requested");
		UpdateRequestNumbersInput(filledRequests);
		return true;
  	end);

	if success then
		LogInfo("Successfully updated Docline batch update form with Filled requests.");
		if (_statusItem ~= nil) then
			_statusItem.Caption = "Yes Requests pasted into Docline window. Submit requests and proceed.";
		end
	else
		LogError("Error updating filled requests: " .. result);
		if (_statusItem ~= nil) then
			_statusItem.Caption = "An error occurred pasting data into docline window.";
		end
	end
end

function GetRejectionReason(reason)

	if (reason == nil) then
		return "";
	end

	--Attempt to handle translating the reason for rejection from previous cancellation codes to newer styles.
	local newReason = reason;
	if (string.lower(reason) == "non circ") then
		newReason = "NCR";
	elseif (string.lower(reason) == "max cost") then
		newReason = "CST";
	elseif (string.lower(reason) == "ordered") then
		newReason = "ORD";
	elseif ((string.lower(reason) == "bad cite") or (string.lower(reason) == "inc*")) then
		newReason = "INC";
	elseif (string.lower(reason) == "lacking") then
		newReason = "LAC";
	elseif (string.lower(reason) == "lost") then
		newReason = "LOS";
	elseif (string.lower(reason) == "bindery") then
		newReason = "BDY";
	end

	if (newReason ~= reason) then
		LogInfo("Modified rejection reason from " .. reason .. " to " .. newReason);
	end

	return newReason;
end
